/* 
 * Skeleton VFS module.  Implements passthrough operation of all VFS
 * calls to disk functions.
 *
 * Copyright (C) Tim Potter, 1999-2000
 * Copyright (C) Alexander Bokovoy, 2002
 * Copyright (C) Stefan (metze) Metzmacher, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Useful stuff:
 *      conn->user
 *      conn->client_address
 *
 * TODO:
 *      fullname not passed to validate()
 *      get params from smb.conf
 *      regex in wanted()
 *
 * BUGS:
 *      validate is flaky - randomly returns true/false
 */

#include "includes.h"

static const char *validate_prog = "/home/rsubr/src/samba-magic/magic/validate";

static int vfs_magic_close(vfs_handle_struct *handle, files_struct *fsp, int fd);
static int vfs_magic_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new);

static BOOL wanted(const char *fname);
static BOOL validate(vfs_handle_struct *handle,  connection_struct *,  const char *fname);

static int vfs_magic_close(vfs_handle_struct *handle, files_struct *fsp, int fd)
{

    char *fname = fsp->fsp_name;


    if (wanted(fname)) {
        if (!validate(handle, handle->conn, fname)) { errno = EACCES; return -1; }
    }

    return SMB_VFS_NEXT_CLOSE(handle, fsp, fd);
}

static int vfs_magic_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new)
{
    if (wanted(new)) {
        if (!validate(handle, conn, old)) { errno = EACCES; return -1; }
    }

    return SMB_VFS_NEXT_RENAME(handle, conn, old, new);
}



static BOOL validate(vfs_handle_struct *handle,  connection_struct *conn,  const char *fname)
{

    int status;
    pid_t pid, status_pid;

    pid = fork();

    switch(pid) {

        case -1:        /* fork failed */
            /* TODO: log event */
            break;


        case 0:         /* child */
            /* TODO: may have to close all fds here */
            execl(validate_prog, validate_prog, fname, (char *) NULL);

            /* if we get here, exec failed so kill child */
            _exit(1);

            break;


        default:        /* parent */
            if (waitpid(pid, &status_pid, 0) == -1) {  /* BUG: waitpid fails at random! */
                syslog(0, "MAGIC: waitpid failed: %s", strerror(errno));
                return False;
            }

            if (WIFEXITED(status) && WEXITSTATUS(status) == 0)
                return True;
    }

    return False;
}


static BOOL wanted(const char *fname)
{
    /* TODO: hardcoded constant, make this a regex match with pattern obtained
     * from smb.conf */

    if (fname && strstr(fname, "xml"))
        return True;

    return False;
}

/* VFS operations structure */

static vfs_op_tuple magic_op_tuples[] = {

	/* File operations */

	{SMB_VFS_OP(vfs_magic_close),			SMB_VFS_OP_CLOSE,		SMB_VFS_LAYER_TRANSPARENT},
	{SMB_VFS_OP(vfs_magic_rename),			SMB_VFS_OP_RENAME,		SMB_VFS_LAYER_TRANSPARENT},
	{NULL,						SMB_VFS_OP_NOOP,			SMB_VFS_LAYER_NOOP}
};

NTSTATUS init_module(void)
{
	return smb_register_vfs(SMB_VFS_INTERFACE_VERSION, "magic", magic_op_tuples);
}
